import unittest
import math
from src.vector2d.vector2d import Vector2D


class TestVector2D(unittest.TestCase):
    def setUp(self):
        self.v1 = Vector2D(3, 4)
        self.v2 = Vector2D(1, 2)

    def test_magnitude(self):
        """Тест вычисления длины вектора"""
        self.assertAlmostEqual(self.v1.magnitude(), 5.0)

    def test_normalized(self):
        """Тест нормализации вектора"""
        v = Vector2D(3, 4)
        norm = v.normalized()
        self.assertAlmostEqual(norm.magnitude(), 1.0)
        self.assertAlmostEqual(norm.x, 0.6)
        self.assertAlmostEqual(norm.y, 0.8)

    def test_dot_product(self):
        """Тест скалярного произведения"""
        result = self.v1.dot(self.v2)
        expected = 3 * 1 + 4 * 2
        self.assertAlmostEqual(result, expected)

    def test_cross_product(self):
        """Тест векторного произведения"""
        result = self.v1.cross(self.v2)
        expected = 3 * 2 - 4 * 1
        self.assertAlmostEqual(result, expected)

    def test_angle(self):
        """Тест вычисления угла"""
        v1 = Vector2D(1, 0)
        v2 = Vector2D(0, 1)
        angle = v1.angle(v2)
        self.assertAlmostEqual(angle, math.pi / 2)

    def test_rotation(self):
        """Тест поворота вектора"""
        v = Vector2D(1, 0)
        rotated = v.rotate(math.pi / 2)
        self.assertAlmostEqual(rotated.x, 0, places=10)
        self.assertAlmostEqual(rotated.y, 1, places=10)

    def test_distance(self):
        """Тест расстояния между векторами"""
        v1 = Vector2D(0, 0)
        v2 = Vector2D(3, 4)
        distance = v1.distance_to(v2)
        self.assertAlmostEqual(distance, 5.0)

    def test_addition(self):
        """Тест сложения векторов"""
        result = self.v1 + self.v2
        self.assertAlmostEqual(result.x, 4)
        self.assertAlmostEqual(result.y, 6)

    def test_scalar_multiplication(self):
        """Тест умножения на скаляр"""
        result = self.v1 * 2
        self.assertAlmostEqual(result.x, 6)
        self.assertAlmostEqual(result.y, 8)

    def test_from_angle(self):
        """Тест создания вектора из угла"""
        v = Vector2D.from_angle(math.pi / 4, math.sqrt(2))
        self.assertAlmostEqual(v.x, 1, places=10)
        self.assertAlmostEqual(v.y, 1, places=10)


if __name__ == '__main__':
    unittest.main()


import math
import numpy as np

class Transform2D:
    def __init__(
            self,
            matrix: np.ndarray | list = np.eye(3, dtype=np.float64)
    ):
        if isinstance(matrix, list):
            self.matrix = np.array(matrix, dtype=np.float64)
        elif isinstance(matrix, np.ndarray):
            self.matrix = matrix.astype(dtype=np.float64)
        else:
            self.matrix = matrix
        if self.matrix.shape != (3, 3):
                raise Exception(f'Invalid matrix dimensions. Expected (3, 3), got {self.matrix.shape}')

    @property
    def a(self): return self.matrix[0, 0]
    @property
    def b(self): return self.matrix[1, 0]
    @property
    def c(self): return self.matrix[0, 1]
    @property
    def d(self): return self.matrix[1, 1]
    @property
    def e(self): return self.matrix[0, 2]
    @property
    def f(self): return self.matrix[1, 2]

    @property
    def scale_x(self): return self.a
    @property
    def scale_y(self): return self.d
    @property
    def shear_x(self): return self.b
    @property
    def shear_y(self): return self.c
    @property
    def translate_x(self): return self.e
    @property
    def translate_y(self): return self.f

    # Конструктор по стандарту svg
    # [[a  c  e
    #   b  d  f
    #   0  0  1]]
    @classmethod
    def svg(
            cls,
            a: float = 1.0,
            b: float = 0.0,
            c: float = 0.0,
            d: float = 1.0,
            e: float = 0.0,
            f: float = 0.0
    ):
        return cls(np.array([
            [a, c, e],
            [b, d, f],
            [0, 0, 1]
        ], dtype=np.float64))

    # Конструктор по трансформации
    @classmethod
    def compose(
            cls,
            translate_x: float  = 0.0,
            translate_y: float  = 0.0,
            scale_x: float      = 1.0,
            scale_y: float      = 1.0,
            angle: float        = 0.0
    ):
        return (
            cls()
            .translate(
                translate_x,
                translate_y,
            )
            .scale(
                scale_x,
                scale_y
            )
            .rotate(angle)
        )

    # Перемещение начала координат на dx и dy
    def translate(self, dx: float, dy: float):
        self.matrix[:2, 2] += [dx, dy]
        return self

    # Отражение
    def reflect(self, rtype: str = 'origin'):
        if rtype == 'origin':
            self.matrix[:2:1, :2:1] *= -1
        elif rtype == 'x':
            self.matrix[1, 1] *= -1
        elif rtype == 'y':
            self.matrix[0, 0] *= -1
        return self

    # Установка начала координат в точку
    def set_origin(self, x: float, y: float):
        self.matrix[:2, 2] = [x, y]
        return self

    # Трансформация по размеру
    def scale(self, scale_x: float, scale_y: float):
        self.matrix[:2:1, :2:1] *= [scale_x, scale_y]
        return self

    # Обратная матрица преобразования
    def inverse(self):
        self.matrix = np.linalg.inv(self.matrix)
        return self

    # Поворот координат
    def rotate(self, angle: float = 0, atype: str = None):
        # Radians default
        theta = 0
        if any(atype == t for t in [None, 'rad']):
            theta = angle
        elif atype == 'deg':
            theta = math.radians(angle)

        self.matrix = np.matmul(
            self.matrix,
            np.array([
                [math.cos(theta), -math.sin(theta), 0],
                [math.sin(theta),  math.cos(theta), 0],
                [              0,                0, 1]
            ], dtype=np.float64)
        )
        return self

    # Установка трансформации
    def set_transform(
            self,
            translate_x,
            translate_y,
            scale_x,
            scale_y,
            angle
    ):
        self.translate(translate_x, translate_y)
        self.scale(scale_x, scale_y)
        self.rotate(angle)
        return self

    # Умножение на правый операнд
    def __mul__(self, other: np.ndarray | list | tuple):
        _dots = np.array(other, dtype=np.float64)
        if len(_dots.shape) > 1:
            dots_len, vec_len = _dots.shape
        else:
            dots_len = 1
            vec_len = _dots.shape

        if vec_len != 2 and _dots.shape != (2,):
            raise Exception(f'Invalid dot({type(_dots)}) dimension. Expected (2,), got {_dots.shape}')
        if dots_len > 1:
            for i in range(dots_len):
                _dots[i] = np.matmul(self.matrix, np.append(_dots[i], 1.0))[:2]
            return _dots
        else:
            return np.matmul(self.matrix, np.append(_dots, 1.0))[:2]

    # Преобразование в строку
    def __str__(self):
        return f'Transform2D({self.a}, {self.b}, {self.c}, {self.d}, {self.e}, {self.f})'

# t = Transform2D.compose(2, 2, 2, 2, math.pi/4)


import unittest
import numpy as np
import math
from src.transform2d.transform2d import Transform2D


class TestTransform2D(unittest.TestCase):

    def setUp(self):
        self.transform = Transform2D()

    def test_creation(self):
        """Тест создания трансформации"""
        t = Transform2D()
        expected_matrix = np.eye(3, dtype=np.float64)
        self.assertTrue(np.array_equal(t.matrix, expected_matrix))

    def test_creation_with_list(self):
        """Тест создания с использованием списка"""
        t = Transform2D([[1, 2, 3], [4, 5, 6], [0, 0, 1]])
        self.assertEqual(t.a, 1)
        self.assertEqual(t.c, 2)
        self.assertEqual(t.e, 3)
        self.assertEqual(t.b, 4)
        self.assertEqual(t.d, 5)
        self.assertEqual(t.f, 6)

    def test_creation_with_numpy_array(self):
        """Тест создания с numpy массивом"""
        t = Transform2D(np.array([[2, 0, 0], [0, 2, 0], [0, 0, 1]]))
        self.assertEqual(t.a, 2)
        self.assertEqual(t.d, 2)

    def test_invalid_matrix_dimensions(self):
        """Тест на неверные размеры матрицы"""
        with self.assertRaises(Exception):
            Transform2D([[1, 2], [3, 4]])

    def test_svg_constructor(self):
        """Тест SVG конструктора"""
        t = Transform2D.svg(a=2, b=1, c=3, d=4, e=5, f=6)
        self.assertEqual(t.a, 2)
        self.assertEqual(t.b, 1)
        self.assertEqual(t.c, 3)
        self.assertEqual(t.d, 4)
        self.assertEqual(t.e, 5)
        self.assertEqual(t.f, 6)

    def test_translate(self):
        """Тест перемещения"""
        t = Transform2D()
        t.translate(10, 20)
        self.assertEqual(t.e, 10)
        self.assertEqual(t.f, 20)

    def test_scale(self):
        """Тест масштабирования"""
        t = Transform2D()
        t.scale(2, 3)
        self.assertEqual(t.a, 2)
        self.assertEqual(t.d, 3)

    def test_rotate_radians(self):
        """Тест поворота в радианах"""
        t = Transform2D()
        t.rotate(math.pi / 2)  # 90 градусов
        expected_cos = math.cos(math.pi / 2)
        expected_sin = math.sin(math.pi / 2)
        self.assertAlmostEqual(t.a, expected_cos, places=10)
        self.assertAlmostEqual(t.b, expected_sin, places=10)
        self.assertAlmostEqual(t.c, -expected_sin, places=10)
        self.assertAlmostEqual(t.d, expected_cos, places=10)

    def test_rotate_degrees(self):
        """Тест поворота в градусах"""
        t = Transform2D()
        t.rotate(90, 'deg')  # 90 градусов
        expected_cos = math.cos(math.radians(90))
        expected_sin = math.sin(math.radians(90))
        self.assertAlmostEqual(t.a, expected_cos, places=10)
        self.assertAlmostEqual(t.b, expected_sin, places=10)

    def test_reflect_origin(self):
        """Тест отражения относительно начала координат"""
        t = Transform2D()
        t.reflect('origin')
        self.assertEqual(t.a, -1)
        self.assertEqual(t.d, -1)

    def test_reflect_x(self):
        """Тест отражения относительно оси X"""
        t = Transform2D()
        t.reflect('x')
        self.assertEqual(t.d, -1)
        self.assertEqual(t.a, 1)

    def test_reflect_y(self):
        """Тест отражения относительно оси Y"""
        t = Transform2D()
        t.reflect('y')
        self.assertEqual(t.a, -1)
        self.assertEqual(t.d, 1)

    def test_inverse(self):
        """Тест обратной матрицы"""
        t = Transform2D.svg(a=2, d=3, e=5)
        original_matrix = t.matrix.copy()
        t.inverse()

        # Произведение матрицы на обратную должно дать единичную матрицу
        result = np.matmul(original_matrix, t.matrix)
        expected = np.eye(3, dtype=np.float64)
        self.assertTrue(np.allclose(result, expected, atol=1e-10))

    def test_transform_single_point(self):
        """Тест преобразования одной точки"""
        t = Transform2D()
        t.translate(10, 20)
        point = [5, 5]
        result = t * point
        expected = [15, 25]
        self.assertTrue(np.allclose(result, expected))

    def test_transform_multiple_points(self):
        """Тест преобразования нескольких точек"""
        t = Transform2D()
        t.scale(2, 2)
        points = [[1, 2], [3, 4], [5, 6]]
        result = t * points
        expected = [[2, 4], [6, 8], [10, 12]]
        self.assertTrue(np.allclose(result, expected))

    def test_compose_method(self):
        """Тест метода compose"""
        t = Transform2D.compose(
            translate_x=10,
            translate_y=20,
            scale_x=2,
            scale_y=3,
            angle=math.pi / 4
        )
        point = [1, 1]
        result = t * point

        # Проверим что преобразование работает
        self.assertEqual(len(result), 2)

    def test_set_transform(self):
        """Тест метода set_transform"""
        t = Transform2D()
        t.set_transform(
            translate_x=10,
            translate_y=20,
            scale_x=2,
            scale_y=3,
            angle=math.pi / 4
        )
        self.assertEqual(t.e, 10)
        self.assertEqual(t.f, 20)

    def test_str_representation(self):
        """Тест строкового представления"""
        t = Transform2D.svg(a=1, b=2, c=3, d=4, e=5, f=6)
        expected_str = "Transform2D(1.0, 2.0, 3.0, 4.0, 5.0, 6.0)"
        self.assertEqual(str(t), expected_str)

    def test_properties(self):
        """Тест свойств трансформации"""
        t = Transform2D.svg(a=2, b=1, c=3, d=4, e=5, f=6)
        self.assertEqual(t.scale_x, 2)
        self.assertEqual(t.scale_y, 4)
        self.assertEqual(t.shear_x, 1)
        self.assertEqual(t.shear_y, 3)
        self.assertEqual(t.translate_x, 5)
        self.assertEqual(t.translate_y, 6)

    def test_transform_chain(self):
        """Тест цепочки преобразований"""
        t = Transform2D()
        t.translate(10, 20).scale(2, 3).rotate(math.pi / 4)
        point = [1, 1]
        result = t * point

        # Проверим что цепочка работает
        self.assertEqual(len(result), 2)
        self.assertIsInstance(result, np.ndarray)

    def test_set_origin(self):
        """Тест установки начала координат"""
        t = Transform2D()
        t.set_origin(100, 200)
        self.assertEqual(t.e, 100)
        self.assertEqual(t.f, 200)


if __name__ == '__main__':
    unittest.main()


import math
import numpy as np


class Vector2D:
    """Класс для работы с 2D векторами"""

    def __init__(self, x: float = 0.0, y: float = 0.0):
        self.x = float(x)
        self.y = float(y)

    @classmethod
    def from_array(cls, arr):
        """Создание вектора из массива"""
        return cls(arr[0], arr[1])

    @classmethod
    def from_angle(cls, angle: float, magnitude: float = 1.0):
        """Создание вектора из угла и величины"""
        return cls(
            magnitude * math.cos(angle),
            magnitude * math.sin(angle)
        )

    def to_array(self):
        """Преобразование в numpy массив"""
        return np.array([self.x, self.y], dtype=np.float64)

    def magnitude(self):
        """Вычисление длины вектора"""
        return math.sqrt(self.x ** 2 + self.y ** 2)

    def normalized(self):
        """Нормализованный вектор"""
        mag = self.magnitude()
        if mag == 0:
            return Vector2D(0, 0)
        return Vector2D(self.x / mag, self.y / mag)

    def dot(self, other):
        """Скалярное произведение"""
        return self.x * other.x + self.y * other.y

    def cross(self, other):
        """Векторное произведение (z-компонента)"""
        return self.x * other.y - self.y * other.x

    def angle(self, other=None):
        """Угол между векторами"""
        if other is None:
            return math.atan2(self.y, self.x)
        dot_product = self.normalized().dot(other.normalized())
        dot_product = max(-1.0, min(1.0, dot_product))  # Ограничение для acos
        return math.acos(dot_product)

    def rotate(self, angle: float):
        """Поворот вектора на угол"""
        cos_a = math.cos(angle)
        sin_a = math.sin(angle)
        return Vector2D(
            self.x * cos_a - self.y * sin_a,
            self.x * sin_a + self.y * cos_a
        )

    def distance_to(self, other):
        """Расстояние до другого вектора"""
        return math.sqrt((self.x - other.x) ** 2 + (self.y - other.y) ** 2)

    # Операторы
    def __add__(self, other):
        return Vector2D(self.x + other.x, self.y + other.y)

    def __sub__(self, other):
        return Vector2D(self.x - other.x, self.y - other.y)

    def __mul__(self, scalar):
        return Vector2D(self.x * scalar, self.y * scalar)

    def __rmul__(self, scalar):
        return self.__mul__(scalar)

    def __truediv__(self, scalar):
        if scalar == 0:
            raise ZeroDivisionError("Division by zero")
        return Vector2D(self.x / scalar, self.y / scalar)

    def __neg__(self):
        return Vector2D(-self.x, -self.y)

    def __eq__(self, other):
        return math.isclose(self.x, other.x) and math.isclose(self.y, other.y)

    def __str__(self):
        return f"Vector2D({self.x:.3f}, {self.y:.3f})"